@@ -1,6 +1,6 @@
 import logging
 
-def setLogging(param: bool, log_level= logging.INFO):
+def setLogging(param: bool, log_level= logging.WARNING):
     if param:
         LOGFORMAT = "  s%(levelname)-8s%(reset)s | %(log_color)s%(message)s%(reset)s"
         stream = logging.StreamHandler()
